==> luna_lambda_tools-0.1.24-py3-none-any.whl: luna_lambda_tools/private/configs/tests/test_perform_migration.py <==
import copy
import re
import subprocess
import unittest
from pathlib import Path

import deepdiff
import ujson
from configs_core.migration import models
from configs_core.migration.generator import generateDump
from configs_core.migration.migrate import MigrationProcessor
from configurator_db_tools.migrations import alembic_config
from sqlalchemy import create_engine, select
from sqlalchemy_utils import create_database, drop_database
from storages.structs.entities import ConfiguratorDBMigration

from configs.dump.services import LUNA_PLATFORM
from configs.settings.all_settings import LUNA_CONFIGURATOR, POSSIBLE_SERVICE_NAMES, SETTINGS
from tests.config import DB_HOST, DB_NAME, DB_PASSWORD, DB_PORT, DB_TYPE, DB_URL, DB_USER

# from configs.settings.all_settings import LUNA_CONFIGURATOR, POSSIBLE_SERVICE_NAMES, SETTINGS
# config path hack to let alembic get connection settings without configurator configs directory attached
CONFIG_FILE = Path(__file__).parent.absolute() / "data" / "config.conf"
CONFIGS_PATH = Path(__file__).parent.parent.absolute()
alembic_config.CONFIG_FILE = CONFIG_FILE


def runBashCommand(bashCommand: str, raiseOnFail: bool) -> tuple[str, str]:
    """
    Run bash command.

    Args:
        bashCommand: bash cmd
        raiseOnFail: whether to raise on cmd fail

    Returns:
        tuple[stdout, stderr]
    """
    res = subprocess.run(bashCommand, shell=True, capture_output=True, cwd=CONFIGS_PATH)
    out, err = (res.stdout or b"").decode(), (res.stderr or b"").decode()
    if res.returncode and raiseOnFail:
        raise AssertionError(f"Command execution failed:\n" f"[STDOUT]:\n{out}\n" f"[STDERR]:\n{err}")
    return out, err


def runMigrations(cmdOptions: str, raiseOnFail: bool = True) -> tuple[str, str]:
    """
    Run config migration.

    Args:
        cmdOptions: options for migrate configs
        raiseOnFail: whether to raise on cmd fail

    Returns:
        tuple[stdout, stderr]
    """
    return runBashCommand(
        f"python3 -m configs.migrate {cmdOptions} --config={CONFIG_FILE}",
        raiseOnFail=raiseOnFail,
    )


def getLimitations(connection):
    """Get limitations from db"""
    selectSt = select([models.Limitation])
    records = connection.execute(selectSt).cursor.fetchall()
    limitations = []
    for record in records:
        limitation = {
            "limitation_name": record[0],
            "validation_schema": ujson.loads(record[1]),
            "services": list(sorted(record[2].split(","))),
            "default_value": ujson.loads(record[3]),
            "description": record[4],
            "group_names": [],
            "external": False,
        }
        if limitation["limitation_name"].startswith("LUNA_CONFIGURATOR"):
            continue
        if "luna-configurator" in limitation["services"]:
            limitation["services"].remove("luna-configurator")

        selectSt = select([models.GroupLimitation.group_name]).where(
            models.GroupLimitation.limitation_name == record[0]
        )
        records = connection.execute(selectSt).cursor.fetchall()
        limitation["group_names"] = sorted([record[0] for record in records])
        limitations.append(limitation)
    return list(sorted(limitations, key=lambda x: x["limitation_name"]))


def getSettings(connection):
    """Get settings from db"""
    selectSt = select([models.Setting])
    records = connection.execute(selectSt).cursor.fetchall()
    settings = []
    for record in records:
        setting = {
            "name": record[5],
            "value": ujson.loads(record[1]),
            "description": record[2],
        }
        if setting["name"].startswith("LUNA_CONFIGURATOR"):
            continue
        selectSt = select([models.Tag.tag]).where(models.Tag.id == record[0])
        records = connection.execute(selectSt).cursor.fetchall()
        setting["tags"] = [record[1] for record in records]
        settings.append(setting)
    return list(sorted(settings, key=lambda x: x["name"]))


def getGroups(connection):
    """Get groups from db"""
    selectSt = select([models.Group])
    records = connection.execute(selectSt).cursor.fetchall()
    groups = [{"group_name": record[0], "description": record[1]} for record in records]
    return list(sorted(groups, key=lambda x: x["group_name"]))


def getDBSettingsDump():
    """
    Get settings dump from db without luna-configurator settings

    Returns:
        Default db dump without configurator settings.
    """
    engine = create_engine(DB_URL)
    with engine.begin() as connection:
        limitations = getLimitations(connection)
        settings = getSettings(connection)
        groups = getGroups(connection)
        return {
            "limitations": limitations,
            "groups": groups,
            "settings": settings,
        }


def getDumpFromFile():
    """
    Get dump from file without luna-configurator settings

    Returns:
        Default configs dump without configurator settings.
    """
    dump = ujson.loads((Path(__file__).parent.parent / "configs" / "migration" / "actual_dump.json").read_text())
    dump = removeConfiguratorFromDump(dump)
    dump.pop("version", None)
    return dump


def getFirstDump():
    """
    Get base revision dump from file

    Returns:
        Base configs dump without configurator settings.
    """
    dump = ujson.loads((Path(__file__).parent.parent / "tests" / "first_dump.json").read_text())
    return removeConfiguratorFromDump(dump)


def removeConfiguratorFromDump(dump: dict):
    """
    Remove configurator settings from dump

    Args:
        dump: configs dump

    Returns:
        Base configs dump without configurator settings.
    """
    limitationPrefixToRemove = "LUNA_CONFIGURATOR"

    dump["limitations"] = [
        lim for lim in dump["limitations"] if not lim["limitation_name"].startswith(limitationPrefixToRemove)
    ]
    for limitation in dump["limitations"].copy():
        if limitation["services"] == ["luna-configurator"]:
            dump["limitations"].remove(limitation)
        elif "luna-configurator" in limitation["services"]:
            limitation["services"].remove("luna-configurator")
        limitation["group_names"] = sorted(limitation["group_names"])

    for setting in dump["settings"].copy():
        if setting["name"].startswith(limitationPrefixToRemove):
            dump["settings"].remove(setting)

    strDump = ujson.dumps(dump, indent=2).lower()
    TARGET_STR = "configurator"
    if TARGET_STR in strDump:
        raise RuntimeError(f"'{TARGET_STR}' must not be found in the dump:\n{strDump}")
    return dump


class TestMigration(unittest.TestCase):
    """
    Test migration

    limitationsPatches: expected differences in limitations and patches for value
    settingsToDefault: settings with expected differences that will be patched to default
    migrationProcessor: migration processor
    dumpFromFile: dump from dump file
    """

    @classmethod
    def setUpClass(cls):
        """Setup"""
        cls.limitationsPatches = {"BACKPORT3_ENABLE_WS_EVENTS": lambda limit: limit.update({"default_value": True})}
        cls.settingsToDefault = [
            ("BACKPORT3_ENABLE_WS_EVENTS", ""),
            ("DEFAULT_FACE_DESCRIPTOR_VERSION", ""),
            ("LUNA_MONITORING", "send_data_for_monitoring"),
            ("LUNA_ACCOUNTS_LOGGER", "log_to_file"),
            ("TASKS_REDIS_DB_ADDRESS", "sentinel.master_name"),
            ("LUNA_VIDEO_AGENT_REDIS_DB_ADDRESS", "sentinel.master_name"),
            ("LUNA_ACCOUNTS_DB", "db_name"),
            (r"LUNA_REMOTE_SDK_(.+)_ESTIMATOR_SETTINGS", "runtime_settings.device_class"),
            (r"LUNA_REMOTE_SDK_(.+)_ESTIMATOR_SETTINGS", "runtime_settings.optimal_batch_size"),
            ("LUNA_REMOTE_SDK_(.+)_DETECTOR_SETTINGS", "runtime_settings.device_class"),
            ("LUNA_REMOTE_SDK_(.+)_DETECTOR_SETTINGS", "runtime_settings.optimal_batch_size"),
            ("(.+)_LOGGER", "log_to_file"),
            ("DEFAULT_HUMAN_DESCRIPTOR_VERSION", ""),
            ("ALLOW_LUNA_ACCOUNT_AUTH_HEADER", ""),
        ]
        cls.dumpFromFile = getDumpFromFile()
        serviceNames = copy.copy(POSSIBLE_SERVICE_NAMES)
        if LUNA_CONFIGURATOR in serviceNames:
            # remove configurator
            serviceNames.remove(LUNA_CONFIGURATOR)
        cls.dumpFromSettings = generateDump(set(serviceNames), SETTINGS)

    def setUp(self):
        create_database(DB_URL)
        settings = {
            "LUNA_CONFIGURATOR_DB": {
                "db_type": DB_TYPE,
                "db_user": DB_USER,
                "db_password": DB_PASSWORD,
                "db_name": DB_NAME,
                "db_host": DB_HOST,
                "db_port": DB_PORT,
            }
        }
        cm = ConfiguratorDBMigration(settings=settings, dbCmd="upgrade", dbRevision="head")
        cm.create()
        self.migrationProcessor = MigrationProcessor(
            DB_URL,
            Path(__file__).parent.parent / "configs" / "migration" / "versions",
            services=LUNA_PLATFORM,
            component="platform",
        )

    def tearDown(self):
        drop_database(DB_URL)

    def _patchSettingsWithDefaults(self, dump: dict):
        """Patch dump settings with default values for settings with expected differences"""

        def pathGet(setting, path):
            value = setting
            for k in path.split("."):
                if not isinstance(value, dict):
                    return value
                value = value.get(k, None)
                if value is None:
                    return None
            return value

        def getDefault(settName: str, path: str):
            default = [lim for lim in dump["limitations"] if lim["limitation_name"] == settName][0]["default_value"]
            return pathGet(default, path)

        def setDefaultValue(setting, path):
            value = getDefault(setting["name"], path)
            if path == "":
                setting["value"] = value
                return
            keys = path.split(".")
            keys = ["value"] + keys
            for key in keys[:-1]:
                setting = setting.setdefault(key, {})
            setting[keys[-1]] = value

        for setting in dump["settings"]:
            for settingToDefault, path in self.settingsToDefault:
                if re.match(settingToDefault, setting["name"]):
                    setDefaultValue(setting, path)
            for limit in dump["limitations"]:
                if limit["limitation_name"] == setting["name"]:
                    setting["description"] = limit["description"]

    def patchDump(self, dump: dict) -> dict:
        """
        Patch dump with default values for expected fields

        Args:
            dump: db dump

        Returns:
            Settings dump with patched fields
        """
        for limitation in dump["limitations"]:
            if limitation["limitation_name"] in self.limitationsPatches:
                self.limitationsPatches[limitation["limitation_name"]](limitation)
        self._patchSettingsWithDefaults(dump)
        return dump

    def test_migration_first_dump(self):
        """Test that db dump and dump from file are equal only with expected differences (migrating from base revision)"""
        firstDump = getFirstDump()
        self.migrationProcessor.applyDump(firstDump)
        firstDump.pop("version")
        runMigrations(cmdOptions="head", raiseOnFail=True)
        dbDump = self.patchDump(getDBSettingsDump())
        diff = deepdiff.DeepDiff(dbDump, self.dumpFromFile)
        self.assertDictEqual(diff, {}, diff)

    def test_migrate_last_revision(self):
        """
        Test default dump can be migrated
            - Apply default dump on empty db
            - Migrate on first revision
            - Migrate on last revision
            - Migrate on previous revision
            - Migrate on last revision
            - Compare dumps
        """
        runMigrations("head", raiseOnFail=True)
        runMigrations("869f0c13", raiseOnFail=True)
        runMigrations("head", raiseOnFail=True)
        initalDump = getDBSettingsDump()
        runMigrations("-1", raiseOnFail=True)
        runMigrations("head", raiseOnFail=True)
        newDump = getDBSettingsDump()
        self.assertDictEqual(initalDump, newDump)

    def test_migrate_with_last_revision(self):
        """Test migrate configs after last revision of dump file."""
        runMigrations(cmdOptions="head", raiseOnFail=True)
        out, err = runMigrations(cmdOptions="head")
        self.assertFalse(err)
        self.assertIn("No migration required, already on", out)

    def test_migrate_invalid_revision(self):
        """Test migrate configs for invalid revision of dump file."""
        invalidRevision = "0b1101000"
        self.migrationProcessor.applyDump(self.dumpFromFile | {"version": invalidRevision})
        _, err = runMigrations(cmdOptions="head", raiseOnFail=False)
        self.assertIn(f"'{invalidRevision}' is not in list", err)

    def test_migration_dump(self):
        """Test that db dump and dump from file are equal only with expected differences"""
        self.maxDiff = None
        runMigrations(cmdOptions="869f0c13", raiseOnFail=True)
        runMigrations(cmdOptions="head", raiseOnFail=True)
        dbDump = self.patchDump(getDBSettingsDump())
        diff = deepdiff.DeepDiff(dbDump, self.dumpFromFile)
        self.assertDictEqual(diff, {}, diff)

==> luna_lambda_tools-0.1.49-py3-none-any.whl: luna_lambda_tools/private/configs/tests/test_perform_migration.py <==
import copy
import re
import subprocess
import unittest
from importlib.util import find_spec
from pathlib import Path

import deepdiff
import ujson
from alembic.config import command
from configs_core.migration import models
from configs_core.migration.generator import generateDump
from configs_core.migration.migrate import MigrationProcessor
from configurator_db_tools.migrations import alembic_config
from sqlalchemy import create_engine, select
from sqlalchemy_utils import create_database, drop_database

from configs.dump.services import LUNA_PLATFORM
from configs.settings.all_settings import LUNA_CONFIGURATOR, POSSIBLE_SERVICE_NAMES, SETTINGS
from tests.config import DB_TYPE, DB_URL

# from configs.settings.all_settings import LUNA_CONFIGURATOR, POSSIBLE_SERVICE_NAMES, SETTINGS
# config path hack to let alembic get connection settings without configurator configs directory attached
CONFIG_FILE = Path(__file__).parent.absolute() / "data" / "config.conf"
CONFIGS_PATH = Path(__file__).parent.parent.absolute()
alembic_config.CONFIG_FILE = CONFIG_FILE
ALEMBIC_DIR = Path(__file__).absolute().parent / "data" / "alembic.ini"


def migrateConfigurator():
    from alembic.config import Config  # pylint: disable=C0415

    spec = find_spec("configurator_db_tools")
    scriptLocation = Path(spec.origin).parent.absolute() / "migrations"

    config = Config(str(ALEMBIC_DIR), attributes={"url": DB_URL, "dbType": DB_TYPE})
    config.set_main_option("script_location", str(scriptLocation))
    config.set_main_option("version_locations", str(scriptLocation / "versions"))
    command.upgrade(config, "head")


def runBashCommand(bashCommand: str, raiseOnFail: bool) -> tuple[str, str]:
    """
    Run bash command.

    Args:
        bashCommand: bash cmd
        raiseOnFail: whether to raise on cmd fail

    Returns:
        tuple[stdout, stderr]
    """
    res = subprocess.run(bashCommand, shell=True, capture_output=True, cwd=CONFIGS_PATH)
    out, err = (res.stdout or b"").decode(), (res.stderr or b"").decode()
    if res.returncode and raiseOnFail:
        raise AssertionError(f"Command execution failed:\n" f"[STDOUT]:\n{out}\n" f"[STDERR]:\n{err}")
    return out, err


def runMigrations(cmdOptions: str, raiseOnFail: bool = True) -> tuple[str, str]:
    """
    Run config migration.

    Args:
        cmdOptions: options for migrate configs
        raiseOnFail: whether to raise on cmd fail

    Returns:
        tuple[stdout, stderr]
    """
    return runBashCommand(
        f"python3 -m configs.migrate {cmdOptions} --config={CONFIG_FILE}",
        raiseOnFail=raiseOnFail,
    )


def getLimitations(connection):
    """Get limitations from db"""
    selectSt = select([models.Limitation])
    records = connection.execute(selectSt).cursor.fetchall()
    limitations = []
    for record in records:
        limitation = {
            "limitation_name": record[0],
            "validation_schema": ujson.loads(record[1]),
            "services": list(sorted(record[2].split(","))),
            "default_value": ujson.loads(record[3]),
            "description": record[4],
            "group_names": [],
            "external": False,
        }
        if limitation["limitation_name"].startswith("LUNA_CONFIGURATOR"):
            continue
        if "luna-configurator" in limitation["services"]:
            limitation["services"].remove("luna-configurator")

        selectSt = select([models.GroupLimitation.group_name]).where(
            models.GroupLimitation.limitation_name == record[0]
        )
        records = connection.execute(selectSt).cursor.fetchall()
        limitation["group_names"] = sorted([record[0] for record in records])
        limitations.append(limitation)
    return list(sorted(limitations, key=lambda x: x["limitation_name"]))


def getSettings(connection):
    """Get settings from db"""
    selectSt = select([models.Setting])
    records = connection.execute(selectSt).cursor.fetchall()
    settings = []
    for record in records:
        setting = {
            "name": record[5],
            "value": ujson.loads(record[1]),
            "description": record[2],
        }
        if setting["name"].startswith("LUNA_CONFIGURATOR"):
            continue
        selectSt = select([models.Tag.tag]).where(models.Tag.id == record[0])
        records = connection.execute(selectSt).cursor.fetchall()
        setting["tags"] = [record[1] for record in records]
        settings.append(setting)
    return list(sorted(settings, key=lambda x: x["name"]))


def getGroups(connection):
    """Get groups from db"""
    selectSt = select([models.Group])
    records = connection.execute(selectSt).cursor.fetchall()
    groups = [{"group_name": record[0], "description": record[1]} for record in records]
    return list(sorted(groups, key=lambda x: x["group_name"]))


def getDBSettingsDump():
    """
    Get settings dump from db without luna-configurator settings

    Returns:
        Default db dump without configurator settings.
    """
    engine = create_engine(DB_URL)
    with engine.begin() as connection:
        limitations = getLimitations(connection)
        settings = getSettings(connection)
        groups = getGroups(connection)
        return {
            "limitations": limitations,
            "groups": groups,
            "settings": settings,
        }


def getDumpFromFile():
    """
    Get dump from file without luna-configurator settings

    Returns:
        Default configs dump without configurator settings.
    """
    dump = ujson.loads((Path(__file__).parent.parent / "configs" / "migration" / "actual_dump.json").read_text())
    dump = removeConfiguratorFromDump(dump)
    dump.pop("version", None)
    return dump


def getFirstDump():
    """
    Get base revision dump from file

    Returns:
        Base configs dump without configurator settings.
    """
    dump = ujson.loads((Path(__file__).parent.parent / "tests" / "first_dump.json").read_text())
    return removeConfiguratorFromDump(dump)


def removeConfiguratorFromDump(dump: dict):
    """
    Remove configurator settings from dump

    Args:
        dump: configs dump

    Returns:
        Base configs dump without configurator settings.
    """
    limitationPrefixToRemove = "LUNA_CONFIGURATOR"

    dump["limitations"] = [
        lim for lim in dump["limitations"] if not lim["limitation_name"].startswith(limitationPrefixToRemove)
    ]
    for limitation in dump["limitations"].copy():
        if limitation["services"] == ["luna-configurator"]:
            dump["limitations"].remove(limitation)
        elif "luna-configurator" in limitation["services"]:
            limitation["services"].remove("luna-configurator")
        limitation["group_names"] = sorted(limitation["group_names"])

    for setting in dump["settings"].copy():
        if setting["name"].startswith(limitationPrefixToRemove):
            dump["settings"].remove(setting)

    strDump = ujson.dumps(dump, indent=2).lower()
    TARGET_STR = "configurator"
    if TARGET_STR in strDump:
        raise RuntimeError(f"'{TARGET_STR}' must not be found in the dump:\n{strDump}")
    return dump


class TestMigration(unittest.TestCase):
    """
    Test migration

    limitationsPatches: expected differences in limitations and patches for value
    settingsToDefault: settings with expected differences that will be patched to default
    migrationProcessor: migration processor
    dumpFromFile: dump from dump file
    """

    @classmethod
    def setUpClass(cls):
        """Setup"""
        cls.limitationsPatches = {"BACKPORT3_ENABLE_WS_EVENTS": lambda limit: limit.update({"default_value": True})}
        cls.settingsToDefault = [
            ("BACKPORT3_ENABLE_WS_EVENTS", ""),
            ("DEFAULT_FACE_DESCRIPTOR_VERSION", ""),
            ("LUNA_MONITORING", "send_data_for_monitoring"),
            ("LUNA_ACCOUNTS_LOGGER", "log_to_file"),
            ("TASKS_REDIS_DB_ADDRESS", "sentinel.master_name"),
            ("LUNA_VIDEO_AGENT_REDIS_DB_ADDRESS", "sentinel.master_name"),
            ("LUNA_ACCOUNTS_DB", "db_name"),
            (r"LUNA_REMOTE_SDK_(.+)_ESTIMATOR_SETTINGS", "runtime_settings.device_class"),
            (r"LUNA_REMOTE_SDK_(.+)_ESTIMATOR_SETTINGS", "runtime_settings.optimal_batch_size"),
            ("LUNA_REMOTE_SDK_(.+)_DETECTOR_SETTINGS", "runtime_settings.device_class"),
            ("LUNA_REMOTE_SDK_(.+)_DETECTOR_SETTINGS", "runtime_settings.optimal_batch_size"),
            ("(.+)_LOGGER", "log_to_file"),
            ("DEFAULT_HUMAN_DESCRIPTOR_VERSION", ""),
            ("ALLOW_LUNA_ACCOUNT_AUTH_HEADER", ""),
        ]
        cls.dumpFromFile = getDumpFromFile()
        serviceNames = copy.copy(POSSIBLE_SERVICE_NAMES)
        if LUNA_CONFIGURATOR in serviceNames:
            # remove configurator
            serviceNames.remove(LUNA_CONFIGURATOR)
        cls.dumpFromSettings = generateDump(set(serviceNames), SETTINGS)

    def setUp(self):
        create_database(DB_URL)
        migrateConfigurator()
        self.migrationProcessor = MigrationProcessor(
            DB_URL,
            Path(__file__).parent.parent / "configs" / "migration" / "versions",
            services=LUNA_PLATFORM,
            component="platform",
        )

    def tearDown(self):
        drop_database(DB_URL)

    def _patchSettingsWithDefaults(self, dump: dict):
        """Patch dump settings with default values for settings with expected differences"""

        def pathGet(setting, path):
            value = setting
            for k in path.split("."):
                if not isinstance(value, dict):
                    return value
                value = value.get(k, None)
                if value is None:
                    return None
            return value

        def getDefault(settName: str, path: str):
            default = [lim for lim in dump["limitations"] if lim["limitation_name"] == settName][0]["default_value"]
            return pathGet(default, path)

        def setDefaultValue(setting, path):
            value = getDefault(setting["name"], path)
            if path == "":
                setting["value"] = value
                return
            keys = path.split(".")
            keys = ["value"] + keys
            for key in keys[:-1]:
                setting = setting.setdefault(key, {})
            setting[keys[-1]] = value

        for setting in dump["settings"]:
            for settingToDefault, path in self.settingsToDefault:
                if re.match(settingToDefault, setting["name"]):
                    setDefaultValue(setting, path)
            for limit in dump["limitations"]:
                if limit["limitation_name"] == setting["name"]:
                    setting["description"] = limit["description"]

    def patchDump(self, dump: dict) -> dict:
        """
        Patch dump with default values for expected fields

        Args:
            dump: db dump

        Returns:
            Settings dump with patched fields
        """
        for limitation in dump["limitations"]:
            if limitation["limitation_name"] in self.limitationsPatches:
                self.limitationsPatches[limitation["limitation_name"]](limitation)
        self._patchSettingsWithDefaults(dump)
        return dump

    def test_migration_first_dump(self):
        """Test that db dump and dump from file are equal only with expected differences (migrating from base revision)"""
        firstDump = getFirstDump()
        self.migrationProcessor.applyDump(firstDump)
        firstDump.pop("version")
        runMigrations(cmdOptions="head", raiseOnFail=True)
        dbDump = self.patchDump(getDBSettingsDump())
        diff = deepdiff.DeepDiff(dbDump, self.dumpFromFile)
        self.assertDictEqual(diff, {}, diff)

    def test_migrate_last_revision(self):
        """
        Test default dump can be migrated
            - Apply default dump on empty db
            - Migrate on first revision
            - Migrate on last revision
            - Migrate on previous revision
            - Migrate on last revision
            - Compare dumps
        """
        runMigrations("head", raiseOnFail=True)
        runMigrations("869f0c13", raiseOnFail=True)
        runMigrations("head", raiseOnFail=True)
        initalDump = getDBSettingsDump()
        runMigrations("-1", raiseOnFail=True)
        runMigrations("head", raiseOnFail=True)
        newDump = getDBSettingsDump()
        self.assertDictEqual(initalDump, newDump)

    def test_migrate_with_last_revision(self):
        """Test migrate configs after last revision of dump file."""
        runMigrations(cmdOptions="head", raiseOnFail=True)
        out, err = runMigrations(cmdOptions="head")
        self.assertFalse(err)
        self.assertIn("No migration required, already on", out)

    def test_migrate_invalid_revision(self):
        """Test migrate configs for invalid revision of dump file."""
        invalidRevision = "0b1101000"
        self.migrationProcessor.applyDump(self.dumpFromFile | {"version": invalidRevision})
        _, err = runMigrations(cmdOptions="head", raiseOnFail=False)
        self.assertIn(f"'{invalidRevision}' is not in list", err)

    def test_migration_dump(self):
        """Test that db dump and dump from file are equal only with expected differences"""
        self.maxDiff = None
        runMigrations(cmdOptions="869f0c13", raiseOnFail=True)
        runMigrations(cmdOptions="head", raiseOnFail=True)
        dbDump = self.patchDump(getDBSettingsDump())
        diff = deepdiff.DeepDiff(dbDump, self.dumpFromFile)
        self.assertDictEqual(diff, {}, diff)
